/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#pragma once
#include <stdbool.h>
#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>

#define min(x, y) ((x) < (y) ? (x) : (y))
#define max(x, y) ((x) > (y) ? (x) : (y))

#define NB_SQUARE 64
enum {
	A1, B1, C1, D1, E1, F1, G1, H1,
	A2, B2, C2, D2, E2, F2, G2, H2,
	A3, B3, C3, D3, E3, F3, G3, H3,
	A4, B4, C4, D4, E4, F4, G4, H4,
	A5, B5, C5, D5, E5, F5, G5, H5,
	A6, B6, C6, D6, E6, F6, G6, H6,
	A7, B7, C7, D7, E7, F7, G7, H7,
	A8, B8, C8, D8, E8, F8, G8, H8,
	NoSquare
};
#define NB_RANK_FILE 8
enum { Rank1, Rank2, Rank3, Rank4, Rank5, Rank6, Rank7, Rank8 };
enum { FileA, FileB, FileC, FileD, FileE, FileF, FileG, FileH };

static inline bool rank_file_ok(unsigned r, unsigned f) { return r < NB_RANK_FILE && f < NB_RANK_FILE; }
static inline unsigned square(unsigned r, unsigned f) { assert(rank_file_ok(r, f)); return 8 * r + f; }
static inline bool square_ok(unsigned sq) { return sq <= H8; }
static inline unsigned rank(unsigned sq) { assert(square_ok(sq)); return sq / 8; }
static inline unsigned file(unsigned sq) { assert(square_ok(sq)); return sq % 8; }

#define NB_PIECE 6
enum { Pawn, Knight, Bishop, Rook, Queen, King, NoPiece };
static inline bool piece_ok(unsigned piece) { return piece < NoPiece; }
static inline bool is_slider(unsigned piece) { assert(piece_ok(piece)); return Bishop <= piece && piece <= Queen; }

#define NB_COLOR 2
enum { White, Black, NoColor };
static inline bool color_ok(unsigned color) { return color <= Black; }
static inline unsigned opp_color(unsigned color) { assert(color_ok(color)); return color ^ 1; }

#define NB_PHASE 2
enum { Opening, EndGame };
static inline bool phase_ok(unsigned phase) { return phase <= EndGame; }

extern const char *PieceLabel[NB_COLOR], *StartFen;

typedef struct {
	uint16_t fsq:6, tsq:6;
	uint16_t promotion:3, ep:1;
} move_t;
#define MAX_MOVES	0x100
static const move_t NoMove = {0,0,0,0};

#define INF	0x7fff
#define MAX_GAME_PLY	0x400	// max nb of plies in a game
#define MAX_PLY			0x80	// max play in the search