/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#include "uci.h"
#include "search.h"

static uint64_t benchmark();
	
int main(int argc, char **argv)
{
	init_bitboard();
	init_psq();
	init_zobrist();
	
	if (argc == 2 && !strcmp(argv[1], "bench")) {
		clock_t start = clock();
		uint64_t nodes = benchmark();
		printf("nodes = %"PRIu64"\ttime = %f\n", nodes, (float)(clock()-start)/CLOCKS_PER_SEC);
	} else
		main_loop();
}

static uint64_t benchmark()
{
	typedef struct {
		char *fen_string;
		int depth;
	} test_position;
	
	static const test_position test[] = {
		{"r1bqkbnr/pp1ppppp/2n5/2p5/4P3/5N2/PPPP1PPP/RNBQKB1R w KQkq -", 12},
		{"r3k2r/p1ppqpb1/bn2pnp1/3PN3/1p2P3/2N2Q1p/PPPBBPPP/R3K2R w KQkq -", 11},
		{"8/2p5/3p4/KP5r/1R3p1k/8/4P1P1/8 w - -", 12},
		{"4rrk1/pp1n3p/3q2pQ/2p1pb2/2PP4/2P3N1/P2B2PP/4RRK1 b - - 7 19", 12},
		{"rq3rk1/ppp2ppp/1bnpb3/3N2B1/3NP3/7P/PPPQ1PP1/2KR3R w - - 7 14", 12},
		{"r1bq1r1k/1pp1n1pp/1p1p4/4p2Q/4Pp2/1BNP4/PPP2PPP/3R1RK1 w - - 2 14", 12},
		{"r3r1k1/2p2ppp/p1p1bn2/8/1q2P3/2NPQN2/PPP3PP/R4RK1 b - - 2 15", 12},
		{"r1bbk1nr/pp3p1p/2n5/1N4p1/2Np1B2/8/PPP2PPP/2KR1B1R w kq - 0 13", 11},
		{"r1bq1rk1/ppp1nppp/4n3/3p3Q/3P4/1BP1B3/PP1N2PP/R4RK1 w - - 1 16", 12},
		{"4r1k1/r1q2ppp/ppp2n2/4P3/5Rb1/1N1BQ3/PPP3PP/R5K1 w - - 1 17", 12},
		{"2rqkb1r/ppp2p2/2npb1p1/1N1Nn2p/2P1PP2/8/PP2B1PP/R1BQK2R b KQ - 0 11", 12},
		{"r1bq1r1k/b1p1npp1/p2p3p/1p6/3PP3/1B2NN2/PP3PPP/R2Q1RK1 w - - 1 16", 11},
		{"3r1rk1/p5pp/bpp1pp2/8/q1PP1P2/b3P3/P2NQRPP/1R2B1K1 b - - 6 22", 12},
		{"r1q2rk1/2p1bppp/2Pp4/p6b/Q1PNp3/4B3/PP1R1PPP/2K4R w - - 2 18", 12},
		{"4k2r/1pb2ppp/1p2p3/1R1p4/3P4/2r1PN2/P4PPP/1R4K1 b - - 3 22", 12},
		{"3q2k1/pb3p1p/4pbp1/2r5/PpN2N2/1P2P2P/5PP1/Q2R2K1 b - - 4 26", 12},
		{"",0}
	};
	Board B;
	uint64_t nodes = 0;
	
	tt_alloc(32 << 20);
	Verbose = false;

	for (int i = 0; test[i].fen_string[0]; i++) {
		printf("fen: %s, depth: %d\n", test[i].fen_string, test[i].depth);
		load_fen(&B, test[i].fen_string);
		id_loop(&B, 0, 0, test[i].depth);
		nodes += node_count;
		tt_clear();
	}
	
	tt_free();
	return nodes;
}
