/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#pragma once
#include "board.h"

bool move_ok(const Board *B, move_t m);

move_t string_to_move(const Board *B, const char *s);
void move_to_string(const Board *B, move_t m, char *s, bool pretty);
unsigned is_check(const Board *B, move_t m);

static inline bool is_OO(const Board *B, move_t m)
{
	return B->piece_on[m.fsq] == King && m.tsq == m.fsq + 2;
}

static inline bool is_OOO(const Board *B, move_t m)
{
	return B->piece_on[m.fsq] == King && m.tsq == m.fsq - 2;
}

static inline bool is_castling(const Board *B, move_t m)
{
	return is_OO(B, m) || is_OOO(B, m);
}

static inline bool is_capture(const Board *B, move_t m)
{
	return B->piece_on[m.tsq] != NoPiece || m.ep;
}

static inline bool is_cop(const Board *B, move_t m)
{
	return is_capture(B, m) || m.promotion != NoPiece;
}

static inline bool move_equal(move_t m1, move_t m2)
{
	return m1.fsq == m2.fsq && m1.tsq == m2.tsq && m1.promotion == m2.promotion && m1.ep == m2.ep;
}

static inline bool is_passed_pawn_push(const Board *B, move_t m)
{
	const unsigned us = B->turn, them = opp_color(us);
	return B->piece_on[m.fsq] == Pawn
		&& test_bit(HalfBoard[them], m.tsq)
		&& !(PassedPawnMask[us][m.tsq] & B->b[them][Pawn])
		&& !(SquaresInFront[us][m.tsq] & B->b[us][Pawn]);
}