/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#include "board.h"

void init_psq();
extern bool PSQInitialized;

extern int PSQTable[NB_PHASE][NB_PIECE][NB_SQUARE];

static inline int get_psqtab(int color, int phase, int piece, int sq)
{
	assert(PSQInitialized && color_ok(color) && phase_ok(phase) && piece_ok(piece) && square_ok(sq));
	return PSQTable[phase][piece][color ? sq : sq ^ 070];
}

bool psq_ok(const Board *B);

/* material values */

#define MP 85
#define EP 100
#define MN 340
#define MB 340
#define MR 550
#define MQ 1100
#define BP 50	// bishop pair

static const unsigned Material[NB_PHASE][NB_PIECE+1] = {
	{MP, MN, MB, MR, MQ, 0, 0},
	{EP, MN, MB, MR, MQ, 0, 0}
};

static inline int optimistic_gain(const Board *B, move_t m)
{
	int result = Material[EndGame][B->piece_on[m.tsq]];
	
	if (m.ep)
		result += EP;
	else if (m.promotion != NoPiece)
		result += Material[EndGame][m.promotion] - MP;
	
	return result;
}
