/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 *
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#pragma once
#include "defs.h"

enum { ScoreExact, ScoreUbound, ScoreLbound };

// Pack key and type into 8 bytes
typedef struct {
	uint64_t packed_key: 62;
	unsigned type: 2;
} pack_t;

// 16 bytes hash entry
typedef struct {
	pack_t pack;
	int16_t eval, score;
	move_t move;
	int8_t depth;
	uint8_t age;
} hash_t;

typedef struct {
	uint64_t count;
	hash_t *buf;
	uint8_t age;
} hash_table_t;

void tt_alloc(uint64_t size);
void tt_clear();
void tt_free();

void tt_newsearch();
void tt_refresh(hash_t *tte);

void tt_write(uint64_t key, int score, int eval, int type, int depth, move_t move);
hash_t *tt_find(uint64_t key);

static inline int hash_type(const hash_t *h) { return h->pack.type; }