/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
 * 
 * Credits: adapted UCI option code from Fruit 2.1
*/
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>
#include "uci_option.h"
#include "util.h"
 
typedef struct {
	const char *var, *init, *type, *extra, *val;
} option_t;

static option_t Option[] = {
	{"Hash", "32", "spin", "min 1 max 8192", NULL},
	{"Verbose", "true", "check", NULL, NULL},
	{NULL, NULL, NULL, NULL, NULL}
};

static option_t *option_find(const char *var);

void option_init()
{
	for (option_t *opt = &Option[0]; opt->var != NULL; opt++)
		option_set(opt->var, opt->init);
}

void option_list()
{
	for (option_t *opt = &Option[0]; opt->var; opt++) {
		if (opt->extra)
			my_send("option name %s type %s default %s %s\n", opt->var, opt->type, opt->val, opt->extra);
		else
			my_send("option name %s type %s default %s\n", opt->var, opt->type, opt->val);
	}
}

bool option_set(const char *var, const char *val)
{
	assert(var && val);
	option_t *opt = option_find(var);

	if (!opt)
		return false;
	else {
		string_set(&opt->val, val);
		return true;
	}
}

const char *option_get_string(const char *var)
{
	assert(var);
	option_t *opt = option_find(var);

	if (!opt)
		my_fatal("option_get(): unknown option \"%s\"\n", var);

	return opt->val;
}

int option_get_int(const char *var)
{
	const char *val = option_get_string(var);
	return atoi(val);
}

bool option_get_bool(const char *var)
{
	const char *val = option_get_string(var);

	if (!strcmp(val,"true") || !strcmp(val,"yes") || !strcmp(val,"1"))
		return true;
	else if (!strcmp(val,"false") || !strcmp(val,"no") || !strcmp(val,"0"))
		return false;
	else {
		assert(false);
		return false;
	}
}

static option_t *option_find(const char *var)
{
	assert(var);

	for (option_t *opt = Option; opt->var != NULL; opt++)
		if (!strcmp(opt->var, var))
			return opt;

	return NULL;
}