/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include "util.h"

#if defined(_WIN32) || defined(_WIN64)
	#include <windows.h>
	#include <conio.h>
#else	// assume POSIX
	#include <errno.h>
	#include <unistd.h>
	#include <sys/time.h>
#endif

void my_send(const char *format, ...)
{
	va_list ap;

	va_start(ap, format);
	vfprintf(stdout, format, ap);
	va_end(ap);
}

void my_fatal(const char *format, ...)
{
	va_list ap;

	va_start(ap, format);
	vfprintf(stderr, format, ap);
	va_end(ap);

	exit(EXIT_FAILURE);
}

char *my_strdup(const char *string)
// strdup() is not ISO C99
{
	assert(string);
	char *result = malloc(strlen(string)+1);
	strcpy(result, string);
	return result;
}

void string_set(const char **variable, const char *string)
{
	assert(variable && string);
	free((void *)(*variable));
	*variable = my_strdup(string);
}

bool input_available()
{
#if defined(_WIN32) || defined(_WIN64)

	static bool init = false, is_pipe;
	static HANDLE stdin_h;
	DWORD val;
	
	if (!init) {
		init = true;
		stdin_h = GetStdHandle(STD_INPUT_HANDLE);
		is_pipe = !GetConsoleMode(stdin_h, &val);
	}

	if (stdin->_cnt > 0)
		return true;
	
	if (is_pipe)
		return !PeekNamedPipe(stdin_h, NULL, 0, NULL, &val, NULL) ? true : val>0;
	else
		return _kbhit();
	
#else	// assume POSIX
	
	fd_set readfds;
	FD_ZERO(&readfds);
	FD_SET(STDIN_FILENO, &readfds);
	
	struct timeval timeout;	
	timeout.tv_sec = timeout.tv_usec = 0;
	
	select(STDIN_FILENO+1, &readfds, 0, 0, &timeout);
	return FD_ISSET(STDIN_FILENO, &readfds);
#endif
}