/*
 * DoubleCheck, a UCI chess playing engine. Copyright (C) 2011 Lucas Braesch.
 * 
 * DoubleCheck is free software: you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * DoubleCheck is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If not,
 * see <http://www.gnu.org/licenses/>.
*/
#pragma once
#include <stdbool.h>

/* my_send is basically a replacement for printf. Somewhat faster, due to the fact that stdout is
 * unbuffered */
void my_send(const char *format, ...);

/* Same as my_send, but terminates the program with exit(EXIT_FAILURE) */
void my_fatal(const char *format, ...);

/* strdup is not ISO C99 compliant, so recode it for portability */
char *my_strdup(const char *string);

/* basically does variable = string, with all the C-style low-level memory (re)allocation involved */
void string_set(const char **variable, const char *string);

/* Tells the polling code whether something is available in stdin for reading. Note that we need to
 * know that WITHOUT actually reading the characters, moving the pointer and flushing the the stdin
 * buffer, which is why that code is OS dependant. Works on POSIX and Windows. */
bool input_available();